 
      Subroutine M05x(F,D1F,RA,RB,D1RA,D1RB,TA,TB,NGrid,ijzy)
************************************************************************
*                                                                      *
*  M06x evaluates the exchange part of the M06 suite of                *
*  functionals on the grid.                                            *
*  !!! Second derivatives are not available yet.                       *
*                                                                      *
c     *  Ref: (a) Zhao, Y., Schultz, N. E., and Truhlar, D. G. J.      *
c     *   Chem. Phys. 123, 161103 (2005).                              *
c     *       (b) Y. Zhao, Schultz, N. E., and D. G. Truhlar, J. Chem. *
c     *   Theory Comput. 2, 364 (2006)                                 * 
c     *                                                                *
c     *       ijzy - 1 M05                                             *
c     *       ijzy - 2 M05-2X                                          *
*                                                                      *
*       ijzy - 1 M06-L                                                 *
*       ijzy - 2 M06-HF                                                *
*       ijzy - 3 M06                                                   *
*       ijzy - 4 M06-2X                                                *
*                                                                      *
c     *  OUTPUT:                                                       *
c     *     F      - Functional values                                 *
c     *     D1F    - First derivatives with respect to RA, RB, GA, GB  *
c     *              TA, TB                                            *
c     *                                                                *
c     *  INPUT:                                                        *
c     *     RA,B   - Spin densities                                    *
c     *     D1RA,B - Spin density gradients                            *
c     *     TA,B   - Spin kinetic energy densities                     *
c     *     NGrid  - number of grids                                   *
*                                                                      *
*  YZ (12/08)                                                          *
*                                                                      *
************************************************************************
      Implicit Real*8(A-H,O-Z)
      INTEGER NGrid
      REAL*8  F(NGrid),D1F(NGrid,7),RA(NGrid),RB(NGrid),
     $        D1RA(NGrid,3),D1RB(NGrid,3),TA(NGrid),TB(NGrid)
      REAL*8 pi

      integer i,j,ijzy
      REAL*8 at1, at2, at3, at4, at5, at6, at7, at8, at9
      REAL*8 at, at10, at11, C1, C2, fL, fNL, at0
      REAL*8 rrho, rho43, rho13, rhoo, rho53
      REAL*8 Gamma2, Gamma
      REAL*8 TauUEG, Tsig, Wsig, W1, W2, W3, W4, W5, W6
      REAL*8 W7, W8, W9, W10, W11, Fsig
      REAL*8 tauu,DTol
      REAL*8 F83, F23, F53, F1o3
      REAL*8 F1o4, F2o3, F3o2, F4o3, F4o9, F3o5
      REAL*8 Nine, F10, F11
      REAL*8  Ax,  x, x2, En, Ed, E, dE, dEn, dEd
      REAL*8 dFdW, dWdT, dTdR, dTdTau, dGGAdR, dFdR
      REAL*8 dFdTau, dGGAdG
      Integer dRA, dRB, dTA, dTB, dGA, dGB
      parameter( pi = 3.1415926535897932384626433832795d0 )
      parameter (F1o3=1.d0/3.d0, F1o4=1.d0/4.d0, F2o3=2.d0/3.d0, 
     &             F3o2=3.d0/2.d0)
      parameter (F4o3=4.d0/3.d0, F4o9=4.0d0/9.0d0, F3o5=3.d0/5.d0)
      parameter (F83=8.0d0/3.0d0, F23=2.0d0/3.d0, F53=5.d0/3.d0)
      parameter (One=1.0d0, Two=2.0d0, Three=3.0d0, Four=4.0d0, 
     &             Five=5.0d0,Six=6.0d0, Seven=7.0d0,
     &             Eight=8.0d0, Nine=9.0d0,F10=10.0d0, F11=11.d0)

      DTol=1.0d-8
      
      if (ijzy.eq.1) then
c     Parameters for M05
        fac = 0.72d0 
        at0=    1.0d0*fac
        at1=    0.08151d0*fac
        at2=    -0.43956d0*fac
        at3=    -3.22422d0*fac
        at4=    2.01819d0*fac
        at5=    8.79431d0*fac
        at6=    -0.00295d0*fac
        at7=    9.82029d0*fac
        at8=    -4.82351d0*fac
        at9=    -48.17574d0*fac
        at10=   3.64802d0*fac
        at11=   34.02248d0*fac
      elseif (ijzy.eq.2) then
c     Parameters for M05-2X
        fac=  0.44d0
        at0=   1.0d0*fac
        at1=    -0.56833d0*fac
        at2=    -1.30057d0*fac
        at3=    5.50070d0*fac
        at4=    9.06402d0*fac
        at5=    -32.21075d0*fac
        at6=    -23.73298d0*fac
        at7=    70.22996d0*fac
        at8=    29.88614d0*fac
        at9=    -60.25778d0*fac
        at10=   -13.22205d0*fac
        at11=   15.23694d0*fac
      endif
*
      

      C1     = 3.36116D-03
      C2     = 4.49267D-03
      
      Ax = -F3o2*(F4o3*PI)**(-F1o3)
      DO i = 1,NGrid
         IF ((RA(i).gt.DTol).and.(TA(i).gt.DTol)) THEN
            rhoo = RA(i)
            rho43 = rhoo**F4o3  
            rrho = 1.0d0/rhoo       ! reciprocal of rho
            rho13 = rho43*rrho
            rho53 = rhoo**F53
c
            tauN = TA(i)
            tauu = tauN
            TauUEG=F3o5*((Six*pi*pi)**F2o3)*rho53
            Tsig =TauUEG/tauN
            Wsig =(Tsig-One)/(Tsig+One)
            Fsig=(at0 + Wsig*(at1 + Wsig*(at2 + Wsig*(at3 + Wsig*(
     &            at4 + Wsig*(at5 + Wsig*(at6 + Wsig*(at7 + Wsig*(
     &            at8 + Wsig*(at9 + Wsig*(at10+Wsig*at11)))))))))))
         Gamma2 =  D1RA(i,1)**2 + D1RA(i,2)**2 + D1RA(i,3)**2
         Gamma = dsqrt(Gamma2)
         x = Gamma/rho43
         x2 = x*x
         En = C1*x2
         Ed = One + C2*x2
         E  = -En/Ed
         F(i)=F(i)+(Ax+E)*Fsig*rho43
c
c     functional derivatives
c

         dEn   = Two*C1*x
         dEd   = Two*C2*x
         dE    = -(dEn*Ed-En*dEd)/(Ed*Ed)
         dFdW=( at1 + Wsig*(Two  *at2 + Wsig*(Three*at3 + Wsig*(
     &            Four *at4 + Wsig*(Five *at5 + Wsig*(Six  *at6 + Wsig*(
     &            Seven*at7 + Wsig*(Eight*at8 + Wsig*(Nine *at9 + Wsig*(
     &            F10  *at10+ Wsig*F11*at11))))))))))
         dWdT = Two/((One + Tsig)**2)
         dTdR = ((Six*PI*PI)**F2o3)*(rhoo**F2o3)/tauu
         dTdTau = -TauUEG/tauu**2
         dGGAdR = F4o3*rho13*(Ax+(E-x*dE))
         dFdR = dFdW*dWdT*dTdR
         dFdTau=dFdW*dWdT*dTdTau
         dGGAdG =(dE/(Two*Gamma))
*        dF/dRhoa
c            D1F(i,GAB) = 0.0d0
*        dF/dRhoa
            D1F(i,dRA) = D1F(i,dRA) + dGGAdR*Fsig + (Ax+E)*rho43*dFdR
*        dF/dGammaaa
            D1F(i,dGA) = D1F(i,dGA) + dGGAdG*Fsig 
*        dF/dTaua
            D1F(i,dTA) = D1F(i,dTA) + rho43*(Ax+E)*dFdTau

         ENDIF
c
c beta component
c
         IF ((RB(i).gt.DTol).and.(TB(i).gt.DTol)) THEN
            rhoo = RB(i)
            rho43 = rhoo**F4o3
            rrho = 1.0d0/rhoo       ! reciprocal of rho
            rho13 = rho43*rrho
            rho53 = rhoo**F53
c
            tauN = TB(i)
            tauu = tauN
            TauUEG=F3o5*((Six*pi*pi)**F2o3)*rho53
            Tsig =TauUEG/tauN
            Wsig =(Tsig-One)/(Tsig+One)
            Fsig=(at0 + Wsig*(at1 + Wsig*(at2 + Wsig*(at3 + Wsig*(
     &            at4 + Wsig*(at5 + Wsig*(at6 + Wsig*(at7 + Wsig*(
     &            at8 + Wsig*(at9 + Wsig*(at10+Wsig*at11)))))))))))
         Gamma2 = D1RB(i,1)**2 + D1RB(i,2)**2 + D1RB(i,3)**2
         Gamma = dsqrt(Gamma2)
         x = Gamma/rho43
         x2 = x*x
         En = C1*x2
         Ed = One + C2*x2
         E  = -En/Ed
         F(i)=F(i)+(Ax+E)*Fsig*rho43
c
c     functional derivatives
c

         dEn   = Two*C1*x
         dEd   = Two*C2*x
         dE    = -(dEn*Ed-En*dEd)/(Ed*Ed)
         dFdW=at*(      at1 + Wsig*(Two  *at2 + Wsig*(Three*at3 + Wsig*(
     &            Four *at4 + Wsig*(Five *at5 + Wsig*(Six  *at6 + Wsig*(
     &            Seven*at7 + Wsig*(Eight*at8 + Wsig*(Nine *at9 + Wsig*(
     &            F10  *at10+ Wsig*F11*at11))))))))))
         dWdT = Two/((One + Tsig)**2)
         dTdR = ((Six*PI*PI)**F2o3)*(rhoo**F2o3)/tauu
         dTdTau = -TauUEG/tauu**2
         dGGAdR = F4o3*rho13*(Ax+(E-x*dE))
         dFdR = dFdW*dWdT*dTdR
         dFdTau=dFdW*dWdT*dTdTau
         dGGAdG =(dE/(Two*Gamma))
c
*        dF/dRhob
           D1F(i,dRB) = D1F(i,dRB) + dGGAdR*Fsig+ (Ax+E)*rho43*dFdR
*        dF/dGammaaa
           D1F(i,dGB) = D1F(i,dGB) + dGGAdG*Fsig
*        dF/dTaua
            D1F(i,dTB) = D1F(i,dTB) + rho43*(Ax+E)*dFdTau

       ENDIF
      enddo
      Return
      End
