      Subroutine M06c(F,D1F,RA,RB,D1RA,D1RB,TA,TB,NGrid,ijzy)

************************************************************************
*                                                                      *
*  CM06 evaluates the M05 correlation part of the M06 suite of         *
*  functionals on the grids.                                           *
*  !!! Second derivatives are not available yet.                       *
*                                                                      *
*  Ref: (a) Zhao, Y.  and Truhlar, D. G. J. Chem. Phys. 125,           *
*    194101 (2006).                                                    *
*       (b) Y. Zhao and D. G. Truhlar, J. Phys. Chem. A (2006),        *
*    110(49),13126-13130.                                              *
*                                                                      *
*       ijzy - 1 M06-L                                                 *
*       ijzy - 2 M06-HF                                                *
*       ijzy - 3 M06                                                   *
*       ijzy - 4 M06-2X                                                *
*       ijzy - 5 revM06-L                                              *
*       ijzy - 6 revM06                                                *
*       ijzy - 7 M06-SX
*       ijzy - 8 M06CR
*                                                                      *
*     *  OUTPUT:                                                       *
*     *     F      - Functional values                                 *
*     *     D1F    - First derivatives with respect to RA, RB, GA, GB  *
*     *              TA, TB                                            *
*     *                                                                *
*     *  INPUT:                                                        *
*     *     RA,B   - Spin densities                                    *
*     *     D1RA,B - Spin density gradients                            *
*     *     TA,B   - Spin kinetic energy densities                     *
*     *     NGrid  - number of grids                                   *
*                                                                      *
*  YZ (12/08)                                                          *
*                                                                      *
************************************************************************
      IMPLICIT NONE

      INTEGER NGrid
      REAL*8  F(NGrid),D1F(NGrid,7),RA(NGrid),RB(NGrid),
     $        D1RA(NGrid,3),D1RB(NGrid,3),TA(NGrid),TB(NGrid)
      REAL*8 Pi, F6, F43, Pi34, F13, 
     &RS,RSP,Zeta,dZdA,dZdB,PotLC,dLdS,dLdZ,d2LdSS,d2LdSZ,d2LdZZ,
     &  P, EUEG,Denom, DenPA, DenPB, DenGA, DenGB
      REAL*8 PA,GAA,TauA,FA,FPA,FGA,FTA,EUA,EUEGA,ChiA,EUPA,ChiAP,ChiAG
      REAL*8 PB,GBB,TauB,FB,FPB,FGB,FTB,EUB,EUEGB,ChiB,EUPB,ChiBP,ChiBG
      REAL*8 sopp0, sopp1,sopp2, sopp3, sopp4
      REAL*8 U, W, dUdChiA,dUdChiB,dUdPA,dUdPB,dUdGA,dUdGB,
     &dWdU,dWdPA,dWdPB, dWdGA,dWdGB,EUEGPA,EUEGPB
      Integer dRA, dRB, dTA, dTB, dGA, dGB

      
      REAL*8  Tol, DTol,F1, F2, F3, F4, COpp
      Data COpp/0.0031d0/,F1/1.0d0/,F2/2.0d0/,F3/3.0d0/,F4/4.0d0/ 


      INTEGER i,j,ijzy

      if (ijzy.eq.1) then
C     Parameters for M06-L Correlation
         sopp0= 6.042374D-01
         sopp1= 1.776783D+02
         sopp2= -2.513252D+02
         sopp3= 7.635173D+01
         sopp4= -1.255699D+01
      elseif (ijzy.eq.2) then
C     Parameters for M06-HF Correlation
         sopp0= 1.674634D+00
         sopp1= 5.732017D+01
         sopp2= 5.955416D+01
         sopp3= -2.311007D+02
         sopp4= 1.255199D+02
      elseif (ijzy.eq.3) then
C     Parameters for M06 Correlation
         sopp0= 3.741539D+00
         sopp1= 2.187098D+02
         sopp2= -4.531252D+02
         sopp3= 2.936479D+02
         sopp4= -6.287470D+01
      elseif (ijzy.eq.4) then
C     Parameters for M06-2X Correlation
         sopp0= 8.833596D-01
         sopp1= 3.357972D+01
         sopp2= -7.043548D+01
         sopp3= 4.978271D+01
         sopp4= -1.852891D+01
      elseif (ijzy.eq.5) then
C     Parameters for revM06-L Correlation
         sopp0= 3.44360696D-01
         sopp1= -5.57080242D-01
         sopp2= -2.009821162D+00
         sopp3= -1.857641887D+00
         sopp4= -1.076639864D+00
      elseif (ijzy.eq.6) then
C     Parameters for revM06 Correlation
         sopp0= 1.222401598D+00
         sopp1= 6.613907336D-01
         sopp2= -1.884581043D+00
         sopp3= -2.780360568D+00
         sopp4= -3.068579344D+00
      elseif (ijzy.eq.7) then
C     Parameters for M06-SX Correlation
         sopp0=  1.63738167314691D-01
         sopp1=  -4.36481171027951D-01
         sopp2=  -1.90232628449712D+00
         sopp3=  -1.42432902881841D+00
         sopp4=  -9.05909137360893D-01
      elseif (ijzy.eq.8) then
C     Parameters for M06CR Correlation
         sopp0=  5.19335960695765D-01
         sopp1=  -2.75471746973256D-01
         sopp2=  -9.62099925538483D-01
         sopp3=  -7.64516552693212D-01
         sopp4=  -3.52377352134486D-01
      endif
      DTol = Tol 
      CALL VS98c(Tol,F,D1F,RA,RB,D1RA,D1RB,TA,TB,NGrid,ijzy+1)
      Pi = F4*ATan(F1)
      F6=6.0d0
      F43 = F4 / F3
      Pi34 = F3 / (F4*Pi)
      F13 = F1 / F3

      DO i = 1,NGrid
         PA = RA(i)
         IF (PA.gt.DTol.and.TA(i).gt.DTol) THEN
            GAA   =  D1RA(i,1)**2 + D1RA(i,2)**2 + D1RA(i,3)**2
            TauA = TA(i)

            call m06css(DTol,PA,GAA,TauA,FA,FPA,FGA,FTA,EUA,
     &                ChiA,EUPA,ChiAP,ChiAG,ijzy)
            F(i) = F(i) + FA
            D1F(i,dRA)  = D1F(i,dRA) + FPA
            D1F(i,dGA) = D1F(i,dGA) + FGA
            D1F(i,dTA)  = D1F(i,dTA) + FTA
         ENDIF
         PB = RB(i)
         IF (PB.gt.DTol.and.TB(i).gt.DTol) THEN
            GBB   =  D1RB(i,1)**2 + D1RB(i,2)**2 + D1RB(i,3)**2
            TauB = TB(i)

            call m06css(DTol,PB,GBB,TauB,FB,FPB,FGB,FTB,EUB,
     &                ChiB,EUPB,ChiBP,ChiBG,ijzy)
            F(i) = F(i) + FB
            D1F(i,dRB)  = D1F(i,dRB) + FPB
            D1F(i,dGB) = D1F(i,dGB) + FGB
            D1F(i,dTB)  = D1F(i,dTB) + FTB
         ENDIF
         P=PA+PB
         IF (PB.gt.DTol.and.PA.gt.DTol) THEN
          RS = (Pi34/P) ** F13 
          RSP = -RS/(F3*P)
          Zeta = (PA-PB)/P
          dZdA = (F1-Zeta)/P
          dZdB = (-F1-Zeta)/P
c       
c      lsdac is a subroutine to evaluate the Perdew-Wang-91 correlation functional 
c      local spin density approximation (LSDA) to the correlation energy of a uniform 
c      electron gas. (Phys. Rev. B 45, 13244 (1992)). Users should provid their own
c      for this LSDA correlation functional or they may find this routine on Kieron 
c      Burke's Web site at http://www.chem.uci.edu/~kieron/dftold2/pubs/PBE.asc
c
          Call lsdac(RS,Zeta,PotLC,dLdS,dLdZ)
          EUEG = P*PotLC - EUA - EUB
          U = COpp*(ChiA+ChiB)/(F1 + COpp*(ChiA+ChiB))
          W = sopp0+U*(sopp1+U*(sopp2+U*(sopp3+U*sopp4)))
          F(i) = F(i) + EUEG*W
          dUdChiA =COpp/(F1 + COpp*(ChiA+ChiB))**2
          dUdChiB =COpp/(F1 + COpp*(ChiA+ChiB))**2
          dUdPA= dUdChiA*ChiAP
          dUdPB= dUdChiB*ChiBP
          dUdGA= dUdChiA*ChiAG
          dUdGB= dUdChiB*ChiBG
          dWdU =sopp1+U*(F2*sopp2+U*(F3*sopp3+U*F4*sopp4))
          dWdPA= dWdU*dUdPA
          dWdPB= dWdU*dUdPB
          dWdGA= dWdU*dUdGA
          dWdGB= dWdU*dUdGB
          EUEGPA = PotLC + P*dLdS*RSP + P*dLdZ*dZdA - EUPA
          EUEGPB = PotLC + P*dLdS*RSP + P*dLdZ*dZdB - EUPB
          D1F(i,dRA) = D1F(i,dRA)+EUEGPA*W + EUEG*dWdPA 
          D1F(i,dGA) = D1F(i,dGA)+EUEG*dWdGA 
          D1F(i,dRB) = D1F(i,dRB)+EUEGPB*W + EUEG*dWdPB  
          D1F(i,dGB) = D1F(i,dGB)+EUEG*dWdGB
         ENDIF
        ENDDO
        
        RETURN
        END 


      Subroutine m06css(DTol,PX,GX,TX,F,FP,FG,FT,EUEG,Chi,EUEGP,
     &                   ChiP,ChiG,ijzy)
      Implicit none
C
C     Compute the same-spin part of the m06 correlation functional for one grid
C     point and one spin-case.
C
C
      integer ijzy
      REAL*8 PX, GX, TX, F, FP, FG, FT, DTol
      REAL*8 EUEG, Chi, EUEGP, ChiP, ChiG
      REAL*8 Zero, Pt25, F1, F2, F3, F4, F5, F6, F8, F11
      REAL*8 ss, sss0,sss1, sss2, sss3, sss4, Css
      REAL*8 Pi, Pi34, F13, F23, F43, F53, F83, F113
      REAL*8 RS, FDUEG, D, Fscc, RSP, dFsccP, dFsccG
      REAL*8 E, W, U, dFsccT, dUdChi, dWdU, dWdP, dWdG 
      REAL*8 d2LdSS,d2LdSZ,d2LdZZ,PotLC,dLdS,dLdZ
     

      Data Zero/0.0d0/, Pt25/0.25d0/, F1/1.0d0/, F2/2.0d0/, F3/3.0d0/,
     $  F4/4.0d0/, F5/5.0d0/, F6/6.0d0/, F8/8.0d0/, F11/11.0d0/,
     $  Css/0.06d0/
C
c      DTol=1.0D-7
      ss=1.0
      if (ijzy.eq.1) then
C     Parameters for M06-L Correlation
         sss0=  5.349466D-01
         sss1=  5.396620D-01
         sss2=  -3.161217D+01
         sss3=  5.149592D+01
         sss4=  -2.919613D+01
      elseif (ijzy.eq.2) then
C     Parameters for M06-HF Correlation
         sss0=  1.023254D-01
         sss1=  -2.453783D+00
         sss2=  2.913180D+01
         sss3=  -3.494358D+01
         sss4=  2.315955D+01
      elseif (ijzy.eq.3) then
C     Parameters for M06 Correlation
         sss0=  5.094055D-01
         sss1=  -1.491085D+00
         sss2=  1.723922D+01
         sss3=  -3.859018D+01
         sss4=  2.845044D+01
      elseif (ijzy.eq.4) then
C     Parameters for M06-2X Correlation
         sss0=  3.097855D-01
         sss1=  -5.528642D+00
         sss2=  1.347420D+01
         sss3=  -3.213623D+01
         sss4=  2.846742D+01
      elseif (ijzy.eq.5) then
C     Parameters for revM06-L Correlation
         sss0=  1.227659748D+00
         sss1=  8.55201283D-01
         sss2=  -3.113346677D+00
         sss3=  -2.239678026D+00
         sss4=  3.54638962D-01
      elseif (ijzy.eq.6) then
C     Parameters for revM06 Correlation
         sss0=  9.017224575D-01
         sss1=  2.079991827D-01
         sss2=  -1.823747562D+00
         sss3=  -1.384430429D+00
         sss4=  -4.423253381D-01
      elseif (ijzy.eq.7) then
C     Parameters for M06-SX Correlation
         sss0=  1.17575011057022D+00
         sss1=  6.58083496678423D-01
         sss2=  -2.78913774852905D+00
         sss3=  -1.18597601856255D+00
         sss4=  1.16439928209688D+00
      elseif (ijzy.eq.8) then
C     Parameters for M06CR Correlation
         sss0=  1.97160087977960D-01
         sss1=  -3.30320389979684D+00
         sss2=  -1.09230982086792D+00
         sss3=  5.04830842824064D-01 
         sss4=  1.09615977988422D+00
      endif
      
      If ((PX.le.DTol))  then
        EUEG = Zero
        Chi = Zero
        EUEGP = Zero
        ChiP = Zero
        ChiG = Zero
        PX = Zero
        GX = Zero 
        TX = Zero
        F  = Zero
        FP = Zero
        FG = Zero
        FT = Zero
      else
        Pi = F4*ATan(F1)
        Pi34 = F3 / (F4*Pi)
        F13 = F1 / F3
        F23 = F2 / F3
        F43 = F2 * F23
        F53 = F5 / F3
        F83 = F8 / F3
        F113 = F11 / F3
        FDUEG = (F3/F5)*(F6*Pi*Pi)**F23
        RS = (Pi34/PX) ** F13
        Call lsdac(RS,F1,PotLC,dLdS,dLdZ)
        EUEG = PX*PotLC
        D = TX - Pt25*GX/PX
C        DUEG = FDUEG*PX**F53
        Chi = GX/PX**F83
        U = Css*Chi/(F1 + Css*Chi)
        W = sss0+U*(sss1+U*(sss2+U*(sss3+U*sss4)))
        Fscc=D/TX
        E = Fscc*W*EUEG
        F = E*ss
        RSP = -RS/(F3*Px)
        ChiG = F1/PX**F83
        ChiP = -F83*Chi/PX
        dFsccP=Pt25*GX/(TX*PX**2)
        dFsccG=-Pt25/(TX*PX)
        dFsccT=Pt25*GX/(PX*TX**2)
        dUdChi=Css/((F1+Css*Chi)**2)
        dWdU=sss1+U*(F2*sss2+U*(F3*sss3+U*F4*sss4))
        dWdP=dWdU*dUdChi*ChiP
        dWdG=dWdU*dUdChi*ChiG 
        EUEGP = PotLC + PX*dLdS*RSP
        FP = ss*(dFsccP*W*EUEG 
     $                 + Fscc*dWdP*EUEG
     $                 + Fscc*W*EUEGP)
        FG = ss*(dFsccG*W*EUEG
     $                 + Fscc*dWdG*EUEG)

        FT = ss*(dFsccT*W*EUEG)
       Endif

       Return
       End


