Subroutine MN15x(F,D1F,RA,RB,D1RA,D1RB,TA,TB,NGrid,ijzy)
**************************************************************
*                                                            *
*  MN15x evaluates the exchange part of MN15-L and MN15      *
*        functionals on the grid.                            *
*                                                            *
*  OUTPUT:                                                   *
*     F   - Functional values                                *
*     D1F - First derivatives with respect to RA, RB, GA, GB *
*              TA, TB                                        *
*                                                            *
*  INPUT:                                                    *
*       ijzy - 1 MN15-L                                      *
*       ijzy - 2 MN15                                        *
*                                                            *
*     RA,B   - Spin densities                                *
*     D1RA,B - Spin density gradients                        *
*     TA,B   - Spin kinetic energy densities                 *
*     NGrid  - number of grids                               *
*                                                            *
*  RP (09/12)                                                *
*                                                            *
**************************************************************
      Implicit Real*8(A-H,O-Z)
c
      INTEGER NGrid
      Integer dRA, dRB, dTA, dTB, dGA, dGB, dGC      
      REAL*8  F(NGrid),D1F(NGrid,7),RA(NGrid),RB(NGrid),
     $        D1RA(NGrid,3),D1RB(NGrid,3),TA(NGrid),TB(NGrid)
      Save One,Two,Three,Four,Five,Six,Seven,Eight,Nine
      Data One/1.0d0/,Two/2.0d0/,Three/3.0d0/,Four/4.0d0/,Five/5.0d0/,
     $  Six/6.0d0/,Seven/7.0d0/,Eight/8.0d0/,Nine/9.0d0/
c
        dRA = 1
        dRB = 2
        dGA = 3
        dGB = 4
        dGC = 5
        dTA = 6
        dTB = 7
c
        G   = 0.004d+0
        ome = 2.5d+0
      If (ijzy.eq.1) then
c
c       MN15-L
c      
        CC000 =  0.670864162d0
        CC001 = -0.822003903d0
        CC002 = -1.022407046d0
        CC003 =  1.689460986d0
        CC004 = -0.00562032d0
        CC005 = -0.110293849d0
        CC010 =  0.972245178d0
        CC011 = -6.697641991d0
        CC012 = -4.322814495d0
        CC013 = -6.786641376d0
        CC014 = -5.687461462d0
        CC020 =  9.419643818d0
        CC021 =  11.83939406d0
        CC022 =  5.086951311d0
        CC023 =  4.302369948d0
        CC030 = -8.07344065d0
        CC031 =  2.429988978d0
        CC032 =  11.09485698d0
        CC100 =  1.247333909d0
        CC101 =  3.700485291d0
        CC102 =  0.867791614d0
        CC103 = -0.591190518d0
        CC104 = -0.295305435d0
        CC110 = -5.825759145d0
        CC111 =  2.537532196d0
        CC112 =  3.143390933d0
        CC113 =  2.939126332d0
        CC120 =  0.599342114d0
        CC121 =  2.241702738d0
        CC122 =  2.035713838d0
        CC200 = -1.525344043d0
        CC201 = -2.325875691d0
        CC202 =  1.141940663d0
        CC203 = -1.563165026d0
        CC210 =  7.882032871d0
        CC211 =  11.93400684d0
        CC212 =  9.852928303d0
        CC300 =  0.584030245d0
        CC301 = -0.720941131d0
        CC302 = -2.836037078d0

      Else If (ijzy.eq.2) then
c
c       MN15
c      
        CC000  =  0.073852235d0
        CC001  = -0.839976156d0
        CC002  = -3.082660125d0
        CC003  = -1.02881285d0
        CC004  = -0.811697255d0
        CC005  = -0.063404387d0
        CC010  =  2.54805518d0
        CC011  = -5.031578906d0
        CC012  =  0.31702159d0
        CC013  =  2.981868205d0
        CC014  = -0.749503735d0
        CC020  =  0.231825661d0
        CC021  =  1.261961411d0
        CC022  =  1.665920815d0
        CC023  =  7.483304941d0
        CC030  = -2.544245723d0
        CC031  =  1.384720031d0
        CC032  =  6.902569885d0
        CC100  =  1.657399451d0
        CC101  =  2.98526709d0
        CC102  =  6.89391326d0
        CC103  =  2.489813993d0
        CC104  =  1.454724691d0
        CC110  = -5.054324071d0
        CC111  =  2.35273334d0
        CC112  =  1.299104132d0
        CC113  =  1.203168217d0
        CC120  =  0.121595877d0
        CC121  =  8.048348238d0
        CC122  =  21.91203659d0
        CC200  = -1.852335832d0
        CC201  = -3.4722735d0
        CC202  = -1.564591493d0
        CC203  = -2.29578769d0
        CC210  =  3.666482991d0
        CC211  =  10.87074639d0
        CC212  =  9.696691388d0
        CC300  =  0.630701064d0
        CC301  = -0.505825216d0
        CC302  = -3.562354535d0

      End If
      Tiny = 1.0d-8
      F12   = Two * Six
      F24   = Four * Six
      F28   = Four * Seven
      F2o3  = Two / Three
      F3o2  = Three / Two
      F1o3  = One / Three
      F4o3  = Four / Three
      F7o3  = Seven / Three
      F8o3  = Eight / Three
      F10o3 = F2o3 * Five
      F28o9 = F28 / Nine
      PI    = Four * ATan(One)
C
C     Local Spin Density factor.
C
      Ax = -F3o2*(F4o3*PI)**(-F1o3)
      Do 10 iG = 1, NGrid
c
c alpha component
c
        If(RA(i).gt.Tiny.and.TA(i).gt.Tiny) then
          F3o5 = Three/Five
          F6   = Six
          F5o3 = Five/Three
          pX    = RA(i)
          GamX2 = D1RA(i)
          S2    = GamX2*pX**(-F8o3)
          U     = G*S2/(One+G*S2)
          E = Ax*pX**F4o3
          Tau    = TA(i)
          TauUEG =F3o5*((F6*PI*PI)**F2o3)*(pX**F5o3) 
          Tsig =TauUEG/Tau
          Wsig =(Tsig - One)/(Tsig + One)
c          
          FU = U
          FV = ome*pX**F1o3/(One+ome*pX**F1o3)
          FT = Wsig          
c
          FMN15 = CC000 + FT*CC001 + FT**2*CC002 + FT**3*CC003 + 
     $     FT**4*CC004 + FT**5*CC005 + 
     $    FU*CC010 + FT*FU*CC011 + FT**2*FU*CC012 + FT**3*FU*CC013 + 
     $     FT**4*FU*CC014 + FU**2*CC020 + 
     $    FT*FU**2*CC021 + FT**2*FU**2*CC022 + FT**3*FU**2*CC023 + 
     $     FU**3*CC030 + FT*FU**3*CC031 + 
     $    FT**2*FU**3*CC032 + FV*CC100 + FV*FT*CC101 + 
     $     FV*FT**2*CC102 + FV*FT**3*CC103 + 
     $    FV*FT**4*CC104 + FV*FU*CC110 + FV*FT*FU*CC111 + 
     $     FV*FT**2*FU*CC112 + FV*FT**3*FU*CC113 + 
     $    FV*FU**2*CC120 + FV*FT*FU**2*CC121 + FV*FT**2*FU**2*CC122 + 
     $     FV**2*CC200 + FV**2*FT*CC201 + 
     $    FV**2*FT**2*CC202 + FV**2*FT**3*CC203 + FV**2*FU*CC210 + 
     $     FV**2*FT*FU*CC211 + 
     $    FV**2*FT**2*FU*CC212 + FV**3*CC300 + FV**3*FT*CC301 + 
     $     FV**3*FT**2*CC302
c
          F(i) = F(i) + E*FMN15
c
          If(MaxDrF.ge.1) then
            ER = F4o3*E/pX
            S    = Sqrt(S2)
            GamX = Sqrt(GamX2)
            SR   = -F4o3*S/pX
            SG   = S/GamX
            US   = Two*G*S/((One+G*S*S)**2)
c         
            dFVdR =  (ome/(Three*pX**F2o3))
     $                *(One+ome*pX**F1o3)**(-Two)
c
            dWdT = Two/((One + Tsig)**2)
            dTdR = ((F6*PI*PI)**F2o3)*(pX**F2o3)/Tau
            dTdTau = -TauUEG/Tau**2
c
            dFTdR = dWdT*dTdR
            dFTdTau=dWdT*dTdTau
c     
          dFMN15dFV = CC100 + FT*CC101 + FT**2*CC102 + FT**3*CC103 + 
     $     FT**4*CC104 + FU*CC110 + 
     $    FT*FU*CC111 + FT**2*FU*CC112 + FT**3*FU*CC113 + 
     $     FU**2*CC120 + FT*FU**2*CC121 + 
     $    FT**2*FU**2*CC122 + Two*FV*CC200 + Two*FV*FT*CC201 + 
     $     Two*FV*FT**2*CC202 + Two*FV*FT**3*CC203 + 
     $    Two*FV*FU*CC210 + Two*FV*FT*FU*CC211+Two*FV*FT**2*FU*CC212+ 
     $     Three*FV**2*CC300 + Three*FV**2*FT*CC301 + 
     $    Three*FV**2*FT**2*CC302
c
          dFMN15dFU = CC010 + FT*CC011 + FT**2*CC012 + FT**3*CC013 + 
     $     FT**4*CC014 + Two*FU*CC020 + 
     $    Two*FT*FU*CC021 + Two*FT**2*FU*CC022 + Two*FT**3*FU*CC023 + 
     $     Three*FU**2*CC030 + Three*FT*FU**2*CC031 + 
     $    Three*FT**2*FU**2*CC032 + FV*CC110 + FV*FT*CC111 + 
     $     FV*FT**2*CC112 + FV*FT**3*CC113 + 
     $    Two*FV*FU*CC120 + Two*FV*FT*FU*CC121+Two*FV*FT**2*FU*CC122+ 
     $     FV**2*CC210 + FV**2*FT*CC211 + 
     $    FV**2*FT**2*CC212
c    
          dFMN15dFT = CC001 + Two*FT*CC002 + Three*FT**2*CC003 + 
     $     Four*FT**3*CC004 + Five*FT**4*CC005 + FU*CC011 + 
     $    Two*FT*FU*CC012 + Three*FT**2*FU*CC013+Four*FT**3*FU*CC014+ 
     $     FU**2*CC021 + Two*FT*FU**2*CC022 + 
     $    Three*FT**2*FU**2*CC023 + FU**3*CC031 + Two*FT*FU**3*CC032 + 
     $     FV*CC101 + Two*FV*FT*CC102 + 
     $    Three*FV*FT**2*CC103 + Four*FV*FT**3*CC104 + FV*FU*CC111 + 
     $     Two*FV*FT*FU*CC112 + Three*FV*FT**2*FU*CC113 + 
     $    FV*FU**2*CC121 + Two*FV*FT*FU**2*CC122 + FV**2*CC201 + 
     $     Two*FV**2*FT*CC202 + 
     $    Three*FV**2*FT**2*CC203 + FV**2*FU*CC211 + 
     $     Two*FV**2*FT*FU*CC212 + FV**3*CC301 + Two*FV**3*FT*CC302
c
        dFMN15dR = dFMN15dFV*dFVdR+dFMN15dFU*US*SR+dFMN15dFT*dFTdR
        dFMN15dG = dFMN15dFU*US*SG
        dFMN15dT = dFMN15dFT*dFTdTau
c
            D1F(i,dRA) = D1F(i,dRA) + ER*FMN15
     $                      + E*dFMN15dR
            D1F(i,dGA) = D1F(i,dGA) + E*dFMN15dG/(Two*GamX)
            D1F(i,dTA) = D1F(i,dTA) + E*dFMN15dT
            endIf
          endIf
c
c beta component
c
        If(RB(i).gt.Tiny.and.TB(i).gt.Tiny) then
          F3o5 = Three/Five
          F6   = Six
          F5o3 = Five/Three
          pX    = RB(i)
          GamX2 = D1RB(i)
          S2    = GamX2*pX**(-F8o3)
          U     = G*S2/(One+G*S2)
          E = Ax*pX**F4o3
          Tau    = TB(i)
          TauUEG =F3o5*((F6*PI*PI)**F2o3)*(pX**F5o3) 
          Tsig =TauUEG/Tau
          Wsig =(Tsig - One)/(Tsig + One)
c          
          FU = U
          FV = ome*pX**F1o3/(One+ome*pX**F1o3)
          FT = Wsig          
c
          FMN15 = CC000 + FT*CC001 + FT**2*CC002 + FT**3*CC003 + 
     $     FT**4*CC004 + FT**5*CC005 + 
     $    FU*CC010 + FT*FU*CC011 + FT**2*FU*CC012 + FT**3*FU*CC013 + 
     $     FT**4*FU*CC014 + FU**2*CC020 + 
     $    FT*FU**2*CC021 + FT**2*FU**2*CC022 + FT**3*FU**2*CC023 + 
     $     FU**3*CC030 + FT*FU**3*CC031 + 
     $    FT**2*FU**3*CC032 + FV*CC100 + FV*FT*CC101 + 
     $     FV*FT**2*CC102 + FV*FT**3*CC103 + 
     $    FV*FT**4*CC104 + FV*FU*CC110 + FV*FT*FU*CC111 + 
     $     FV*FT**2*FU*CC112 + FV*FT**3*FU*CC113 + 
     $    FV*FU**2*CC120 + FV*FT*FU**2*CC121 + FV*FT**2*FU**2*CC122 + 
     $     FV**2*CC200 + FV**2*FT*CC201 + 
     $    FV**2*FT**2*CC202 + FV**2*FT**3*CC203 + FV**2*FU*CC210 + 
     $     FV**2*FT*FU*CC211 + 
     $    FV**2*FT**2*FU*CC212 + FV**3*CC300 + FV**3*FT*CC301 + 
     $     FV**3*FT**2*CC302
c
          F(i) = F(i) + E*FMN15
c
          If(MaxDrF.ge.1) then
            ER = F4o3*E/pX
            S    = Sqrt(S2)
            GamX = Sqrt(GamX2)
            SR   = -F4o3*S/pX
            SG   = S/GamX
            US   = Two*G*S/((One+G*S*S)**2)
c         
            dFVdR =  (ome/(Three*pX**F2o3))
     $                *(One+ome*pX**F1o3)**(-Two)
c
            dWdT = Two/((One + Tsig)**2)
            dTdR = ((F6*PI*PI)**F2o3)*(pX**F2o3)/Tau
            dTdTau = -TauUEG/Tau**2
c
            dFTdR = dWdT*dTdR
            dFTdTau=dWdT*dTdTau
c     
          dFMN15dFV = CC100 + FT*CC101 + FT**2*CC102 + FT**3*CC103 + 
     $     FT**4*CC104 + FU*CC110 + 
     $    FT*FU*CC111 + FT**2*FU*CC112 + FT**3*FU*CC113 + 
     $     FU**2*CC120 + FT*FU**2*CC121 + 
     $    FT**2*FU**2*CC122 + Two*FV*CC200 + Two*FV*FT*CC201 + 
     $     Two*FV*FT**2*CC202 + Two*FV*FT**3*CC203 + 
     $    Two*FV*FU*CC210 + Two*FV*FT*FU*CC211+Two*FV*FT**2*FU*CC212+ 
     $     Three*FV**2*CC300 + Three*FV**2*FT*CC301 + 
     $    Three*FV**2*FT**2*CC302
c
          dFMN15dFU = CC010 + FT*CC011 + FT**2*CC012 + FT**3*CC013 + 
     $     FT**4*CC014 + Two*FU*CC020 + 
     $    Two*FT*FU*CC021 + Two*FT**2*FU*CC022 + Two*FT**3*FU*CC023 + 
     $     Three*FU**2*CC030 + Three*FT*FU**2*CC031 + 
     $    Three*FT**2*FU**2*CC032 + FV*CC110 + FV*FT*CC111 + 
     $     FV*FT**2*CC112 + FV*FT**3*CC113 + 
     $    Two*FV*FU*CC120 + Two*FV*FT*FU*CC121+Two*FV*FT**2*FU*CC122+ 
     $     FV**2*CC210 + FV**2*FT*CC211 + 
     $    FV**2*FT**2*CC212
c    
          dFMN15dFT = CC001 + Two*FT*CC002 + Three*FT**2*CC003 + 
     $     Four*FT**3*CC004 + Five*FT**4*CC005 + FU*CC011 + 
     $    Two*FT*FU*CC012 + Three*FT**2*FU*CC013+Four*FT**3*FU*CC014+ 
     $     FU**2*CC021 + Two*FT*FU**2*CC022 + 
     $    Three*FT**2*FU**2*CC023 + FU**3*CC031 + Two*FT*FU**3*CC032 + 
     $     FV*CC101 + Two*FV*FT*CC102 + 
     $    Three*FV*FT**2*CC103 + Four*FV*FT**3*CC104 + FV*FU*CC111 + 
     $     Two*FV*FT*FU*CC112 + Three*FV*FT**2*FU*CC113 + 
     $    FV*FU**2*CC121 + Two*FV*FT*FU**2*CC122 + FV**2*CC201 + 
     $     Two*FV**2*FT*CC202 + 
     $    Three*FV**2*FT**2*CC203 + FV**2*FU*CC211 + 
     $     Two*FV**2*FT*FU*CC212 + FV**3*CC301 + Two*FV**3*FT*CC302
c
        dFMN15dR = dFMN15dFV*dFVdR+dFMN15dFU*US*SR+dFMN15dFT*dFTdR
        dFMN15dG = dFMN15dFU*US*SG
        dFMN15dT = dFMN15dFT*dFTdTau
c
            D1F(i,dRB) = D1F(i,dRB) + ER*FMN15
     $                      + E*dFMN15dR
            D1F(i,dGB) = D1F(i,dGB) + E*dFMN15dG/(Two*GamX)
            D1F(i,dTB) = D1F(i,dTB) + E*dFMN15dT
            endIf
          endIf
   10   Continue
      Return
      End
