      Subroutine SOGGA11C(F,D1F,RA,RB,D1RA,D1RB,NGrid,ijzy)
      Implicit Real*8(A-H,O-Z)
************************************************************************
*                                                                      *
*  SOGGA11C evaluates the correlation part of the SOGGA11 and SOGGA11-X*
*  functionals on the grid.                                            *
*                                                                      *
*    OUTPUT:                                                           *
*       F      - Functional values                                     *
*       D1F    - First derivatives with respect to RA, RB, GA, GB      *
*                                                                      *
*    INPUT:                                                            *
*       ijzy = 1 - SOGGA11 functional                                  *
*       ijzy = 2 - SOGGA11-X functional                                *
*       RA,B     - Spin densities                                      *
*       D1RA,B   - Spin density gradients                              *
*       NGrid    - number of grid points                               *
*                                                                      *
*  Analytic second derivatives are not available yet!!!                *
*                                                                      *
*  RP (11/11)                                                          *
*                                                                      *
************************************************************************
      INTEGER NGrid
      Integer dRA, dRB, dGA, dGB, dGC
      REAL*8  F(NGrid),D1F(NGrid,5),RA(NGrid),RB(NGrid),
     $        D1RA(NGrid,3),D1RB(NGrid,3),CcA(0:5),CcB(0:5)
c
      Save F0,F1,F2,F3,F4,F5,F6,F7,F8,F9,F10
      Data F0/0.0d0/,F1/1.0d0/,F2/2.0d0/,F3/3.0d0/,F4/4.0d0/,F5/5.0d0/,
     $     F6/6.0d0/,F7/7.0d0/,F8/8.0d0/,F9/9.0d0/,F10/10.0d0/
c
      dRA = 1
      dRB = 2
      dGA = 3
      dGB = 4
      dGC = 5
c
      TinyDFT = 1.0d-12
c
      if (ijzy.eq.1) then
       CcA(0) =  0.50000D+00
       CcA(1) = -4.62334D+00
       CcA(2) =  8.00410D+00
       CcA(3) = -130.226D+00
       CcA(4) =  38.2685D+00
       CcA(5) =  69.5599D+00
       CcB(0) =  0.50000D+00
       CcB(1) =  3.62334D+00
       CcB(2) =  9.36393D+00
       CcB(3) =  34.5114D+00
       CcB(4) = -18.5684D+00
       CcB(5) = -0.16519D+00
      elseif (ijzy.eq.2) then
       CcA(0) =  5.00000d-01
       CcA(1) =  7.82439d+01
       CcA(2) =  2.57211d+01
       CcA(3) = -1.38830d+01
       CcA(4) = -9.87375d+00
       CcA(5) = -1.41357d+01
       CcB(0) =  5.00000d-01
       CcB(1) = -7.92439d+01
       CcB(2) =  1.63725d+01
       CcB(3) =  2.08129d+00
       CcB(4) =  7.50769d+00
       CcB(5) = -1.01861d+01
      endif
c
      Pi = ACos(-F1)
      Pi34 = F3/(F4*Pi)
      F1o2 = F1/F2
      F1o3 = F1/F3
      F1o6 = F1/F6
      F2o3 = F2/F3
      F3o5 = F3/F5
      F5o3 = F5/F3
      F7o6 = F7/F6
      XNu = 15.75592D0
      CC0 = 0.004235D0
      beta= XNu*CC0   
      SCfac=F1
c
      Do 100 i = 1, nGrid
       RhoA = RA(i)
       RhoB = RB(i)
       Rho = RhoA + RhoB
c
        GA = (D1RA(i,1) + D1RB(i,1))**F2
        GB = (D1RA(i,2) + D1RB(i,2))**F2
        GC = (D1RA(i,3) + D1RB(i,3))**F2
        YY = GA + GB + GC
        GRho = Sqrt(YY)
c
       If(Rho.gt.TinyDFT) then
          rS = (Pi34/Rho)**F1o3
          Zeta = (RhoA-RhoB)/Rho
c       
c      lsdac is a subroutine to evaluate the Perdew-Wang-91 correlation functional 
c      local spin density approximation (LSDA) to the correlation energy of a uniform 
c      electron gas. (Phys. Rev. B 45, 13244 (1992)). Users should provid their own
c      for this LSDA correlation functional or they may find this routine on Kieron 
c      Burke's Web site at http://www.chem.uci.edu/~kieron/dftold2/pubs/PBE.asc
c
          Call lsdac(RS,Zeta,PotLC,dLdS,dLdZ)
c       
c      GZeta is a subroutine to evaluate the function G(Z) and its derivatives.
c      Users should provid their own routine or they may find this routine as 
c      part of the PBEH0 routine on Kieron Burke's Web site 
c      at http://www.chem.uci.edu/~kieron/dftold2/pubs/PBE.asc
c
          Call GZeta(Zeta,G,dGdZ,dGdZZ,dGdZZZ)
c
          Eloc = SCfac*Rho*PotLC
          FKFac = (F3*Pi*Pi)**F1o3
          SKFac = Sqrt(FKFac/Pi)*F2
          Rho16 = Rho**F1o6
          Rho76 = Rho*Rho16
          T  = GRho/(F2*SKFac*Rho76*G)
          T2 = T*T
          G2 = G*G
          G3 = G*G2
          PON = (G3*beta*T2)/PotLC
          Ffrac = F1-F1/(F1-PON)
          Fexp  = F1-exp(PON)
          fa0 = CcA(0)
          fa1 = CcA(1) *Ffrac
          fa2 = CcA(2) *Ffrac**F2
          fa3 = CcA(3) *Ffrac**F3
          fa4 = CcA(4) *Ffrac**F4
          fa5 = CcA(5) *Ffrac**F5
          fb0 = CcB(0)
          fb1 = CcB(1) *Fexp
          fb2 = CcB(2) *Fexp**F2
          fb3 = CcB(3) *Fexp**F3
          fb4 = CcB(4) *Fexp**F4
          fb5 = CcB(5) *Fexp**F5
          FggaC = fa0+fa1+fa2+fa3+fa4+fa5+
     $            fb0+fb1+fb2+fb3+fb4+fb5
c
           F(i) = F(i) + Eloc*FggaC
C
C     1st derivatives.
C
          dSdR  = -(F1o3*rS/Rho)
          dZdA = (F1-Zeta)/Rho
          dZdB = (-F1-Zeta)/Rho
          dLdA = dLdS*dSdR + dLdz*dZdA
          dLdB = dLdS*dSdR + dLdz*dZdB
          dElocdA = SCfac*PotLC + SCfac*Rho*dLdA
          dElocdB = SCfac*PotLC + SCfac*Rho*dLdB
          dT2dR = -F2*F7o6*T2/Rho
          dT2dG =  F2*T2/GRho
          dT2dZ = -F2*dGdZ*T2/G
          dPONdR = -(dLdS*dSdR*PON/PotLC)+dT2dR*PON/T2
          dPONdG = dT2dG*PON/T2
          dPONdZ = F3*dGdZ*PON/G-dLdZ*PON/PotLC+dT2dZ*PON/T2
          dFfracdPON = -F1/((F1-PON)**F2)
          dFexpdPON  = -exp(PON)
          dfa1dR = CcA(1)                 *dFfracdPON*dPONdR
          dfa2dR = CcA(2) *(F2 *Ffrac)    *dFfracdPON*dPONdR
          dfa3dR = CcA(3) *(F3 *Ffrac**F2)*dFfracdPON*dPONdR
          dfa4dR = CcA(4) *(F4 *Ffrac**F3)*dFfracdPON*dPONdR
          dfa5dR = CcA(5) *(F5 *Ffrac**F4)*dFfracdPON*dPONdR
          dfa1dG = CcA(1)                 *dFfracdPON*dPONdG
          dfa2dG = CcA(2) *(F2 *Ffrac)    *dFfracdPON*dPONdG
          dfa3dG = CcA(3) *(F3 *Ffrac**F2)*dFfracdPON*dPONdG
          dfa4dG = CcA(4) *(F4 *Ffrac**F3)*dFfracdPON*dPONdG
          dfa5dG = CcA(5) *(F5 *Ffrac**F4)*dFfracdPON*dPONdG
          dfa1dZ = CcA(1)                 *dFfracdPON*dPONdZ
          dfa2dZ = CcA(2) *(F2 *Ffrac)    *dFfracdPON*dPONdZ
          dfa3dZ = CcA(3) *(F3 *Ffrac**F2)*dFfracdPON*dPONdZ
          dfa4dZ = CcA(4) *(F4 *Ffrac**F3)*dFfracdPON*dPONdZ
          dfa5dZ = CcA(5) *(F5 *Ffrac**F4)*dFfracdPON*dPONdZ
          dfb1dR = CcB(1)                 *dFexpdPON*dPONdR
          dfb2dR = CcB(2) *(F2 *Fexp)     *dFexpdPON*dPONdR
          dfb3dR = CcB(3) *(F3 *Fexp**F2) *dFexpdPON*dPONdR
          dfb4dR = CcB(4) *(F4 *Fexp**F3) *dFexpdPON*dPONdR
          dfb5dR = CcB(5) *(F5 *Fexp**F4) *dFexpdPON*dPONdR
          dfb1dG = CcB(1)                 *dFexpdPON*dPONdG
          dfb2dG = CcB(2) *(F2 *Fexp)     *dFexpdPON*dPONdG
          dfb3dG = CcB(3) *(F3 *Fexp**F2) *dFexpdPON*dPONdG
          dfb4dG = CcB(4) *(F4 *Fexp**F3) *dFexpdPON*dPONdG
          dfb5dG = CcB(5) *(F5 *Fexp**F4) *dFexpdPON*dPONdG
          dfb1dZ = CcB(1)                 *dFexpdPON*dPONdZ
          dfb2dZ = CcB(2) *(F2 *Fexp)     *dFexpdPON*dPONdZ
          dfb3dZ = CcB(3) *(F3 *Fexp**F2) *dFexpdPON*dPONdZ
          dfb4dZ = CcB(4) *(F4 *Fexp**F3) *dFexpdPON*dPONdZ
          dfb5dZ = CcB(5) *(F5 *Fexp**F4) *dFexpdPON*dPONdZ
c
          dFggaCdR = dfa1dR+dfa2dR+dfa3dR+dfa4dR+dfa5dR +
     $               dfb1dR+dfb2dR+dfb3dR+dfb4dR+dfb5dR 
          dFggaCdG = dfa1dG+dfa2dG+dfa3dG+dfa4dG+dfa5dG +
     $               dfb1dG+dfb2dG+dfb3dG+dfb4dG+dfb5dG 
          dFggaCdZ = dfa1dZ+dfa2dZ+dfa3dZ+dfa4dZ+dfa5dZ +
     $               dfb1dZ+dfb2dZ+dfb3dZ+dfb4dZ+dfb5dZ 
c
          dFggaCdA = dFggaCdR + dFggaCdZ*dZdA
          dFggaCdB = dFggaCdR + dFggaCdZ*dZdB
          dGdGx = F1 / (F2*GRho)
          dFggaCdGx = dFggaCdG*dGdGx
C          
          D1F(i,dRA) = D1F(i,dRA) + dElocdA*FggaC + 
     $               Eloc*dFggaCdA
          D1F(i,dRB) = D1F(i,dRB) + dElocdB*FggaC +
     $               Eloc*dFggaCdB
c            
          D1F(i,dGA) = D1F(i,dGA) +    Eloc*dFggaCdGx
          D1F(i,dGB) = D1F(i,dGB) +    Eloc*dFggaCdGx
C     GC is the dot product of the vectors D1RA and D1RB
          D1F(i,dGC) = D1F(i,dGC) + F2*Eloc*dFggaCdGx

        endIf
  100 Continue
      Return
      End
