      Subroutine SOGGA11X(F,D1F,RA,RB,D1RA,D1RB,NGrid,ijzy)
      Implicit Real*8(A-H,O-Z)
************************************************************************
*                                                                      *
*  SOGGA11X evaluates the exchange part of the SOGGA, SOGGA11          *
*  and SOGGA11-X functionals on the grid.                              *
*                                                                      *
*     a) Zhao and Truhlar, J.Chem.Phys., 128, 184109 (2008)            *
*     b) Peverati, Zhao and Truhlar, J.Phys.Chem.Lett, 2, 1991 (2011)  *
*     c) Peverati and Truhlar, J.Chem.Phys, 135, 191102 (2011)         *
*                                                                      *
*    OUTPUT:                                                           *
*      F      - Functional values                                      *
*      D1F    - First derivatives with respect to RA, RB, GA, GB       *
*                                                                      *
*    INPUT:                                                            *
*      ijzy = 1 - SOGGA functional (a) - it requres PBE correlation    *
*      ijzy = 2 - SOGGA11 functional (b)                               *
*      ijzy = 3 - SOGGA11-X functional (c)                             *
*      RA,B     - Spin densities                                       *
*      D1RA,B   - Spin density gradients                               *
*      NGrid    - number of grid points                                *
*                                                                      *
*  Analytic second derivatives are not available yet!!!                *
*                                                                      *
*  RP (11/11)                                                          *
*                                                                      *
************************************************************************
      INTEGER NGrid
      Integer dRA, dRB, dGA, dGB, dGC
      Real*8 RA(*), RB(*), D1RA(NGrid,3), D1RB(NGrid,3),
     $  F(*), D1F(NGrid,*),Mu
C
      Save F0,F1,F2,F3,F4,F5,F6,F7,F8,F9,F10,F48,F81
      Data F0/0.0d0/,F1/1.0d0/,F2/2.0d0/,F3/3.0d0/,F4/4.0d0/,F5/5.0d0/,
     $     F6/6.0d0/,F7/7.0d0/,F8/8.0d0/,F9/9.0d0/,F10/10.0d0/,
     $     F48/48.0d0/,F81/81.0d0/
c
      dRA = 1
      dRB = 2
      dGA = 3
      dGB = 4
      dGC = 5
c
      if (ijzy.eq.1) then
c SOGGA
       CxA0 = 0.5d0
       CxA1 = 0.276d0
       CxA2 = 0d0
       CxA3 = 0d0
       CxA4 = 0d0
       CxA5 = 0d0
       CxB0 = 0.5d0
       CxB1 = 0.276d0
       CxB2 = 0d0
       CxB3 = 0d0
       CxB4 = 0d0
       CxB5 = 0d0
      elseif (ijzy.eq.2) then
c SOGGA11
       CxA0 =  0.50000d0
       CxA1 = -2.95535d0
       CxA2 =  15.7974d0
       CxA3 = -91.1804d0
       CxA4 =  96.2030d0
       CxA5 =  0.18683d0
       CxB0 =  0.50000d0
       CxB1 =  3.50743d0
       CxB2 = -12.9523d0
       CxB3 =  49.7870d0
       CxB4 = -33.2545d0
       CxB5 = -11.1396d0
      elseif (ijzy.eq.3) then
c SOGGA11-X
       CxA0 =  2.99250d-01
       CxA1 =  3.21638d+00
       CxA2 = -3.55605d+00
       CxA3 =  7.65852d+00
       CxA4 = -1.12830d+01
       CxA5 =  5.25813d+00
       CxB0 =  2.99250d-01
       CxB1 = -2.88595d+00
       CxB2 =  3.23617d+00
       CxB3 = -2.45393d+00
       CxB4 = -3.75495d+00
       CxB5 =  3.96613d+00
      endif
c
      TinyDFT = 1.0d-8
      F1o3 = F1/F3 
      F2o3 = F2/F3
      F3o5 = F3/F5
      F4o3 = F4/F3
      F5o3 = F5/F3
      Pi   = ACos(-F1)
      AsO  = (F48*PI*PI)**F1o3
      As   = F1/AsO
      Ax   = -(F3/F2) * (F4o3*Pi)**(-F1o3)
      mu = 0.2236536053d0
c
      Do 100 iG = 1, NGrid
        If(RA(iG).gt.TinyDFT) Then
          Rho = RA(iG)
          Gam = D1RA(iG,1)**F2 + D1RA(iG,2)**F2 + D1RA(iG,3)**F2
          Rho43 = Rho**F4o3
          Gam12 = Sqrt(Gam)
c
          Eloc = Ax*Rho43
          x = Gam12/Rho43
          s = As*x
          y = s*s
          PON = mu*y
          Ffrac = F1-F1/(F1+PON)
          Fexp  = F1-exp(-PON)
          fa0 = CxA0
          fa1 = CxA1 *Ffrac
          fa2 = CxA2 *Ffrac**F2
          fa3 = CxA3 *Ffrac**F3
          fa4 = CxA4 *Ffrac**F4
          fa5 = CxA5 *Ffrac**F5
          fb0 = CxB0
          fb1 = CxB1 *Fexp
          fb2 = CxB2 *Fexp**F2
          fb3 = CxB3 *Fexp**F3
          fb4 = CxB4 *Fexp**F4
          fb5 = CxB5 *Fexp**F5
c
          Fggax = fa0+fa1+fa2+fa3+fa4+fa5 +
     $            fb0+fb1+fb2+fb3+fb4+fb5
c 
          F(iG)=F(iG) + Eloc*Fggax
C
C     1st derivatives.
C
            Rho13  = Rho**F1o3
            dElocdR=Ax*F4o3*Rho13
            dydR = -(F8/F3)*y/Rho
            dydG   = y/Gam
            dFfracdPON = F1/((F1+PON)**F2)
            dFexpdPON  = exp(-PON)
            dfa1dy = CxA1 *mu*dFfracdPON
            dfa2dy = CxA2 *mu*F2*Ffrac*dFfracdPON
            dfa3dy = CxA3 *mu*(F3 *Ffrac**F2)*dFfracdPON
            dfa4dy = CxA4 *mu*(F4 *Ffrac**F3)*dFfracdPON
            dfa5dy = CxA5 *mu*(F5 *Ffrac**F4)*dFfracdPON
            dfa1dR = dfa1dy *dydR
            dfa2dR = dfa2dy *dydR
            dfa3dR = dfa3dy *dydR
            dfa4dR = dfa4dy *dydR
            dfa5dR = dfa5dy *dydR
            dfa1dG = dfa1dy *dydG
            dfa2dG = dfa2dy *dydG
            dfa3dG = dfa3dy *dydG
            dfa4dG = dfa4dy *dydG
            dfa5dG = dfa5dy *dydG
            dfb1dy = CxB1 *mu*dFexpdPON
            dfb2dy = CxB2 *mu*F2*Fexp*dFexpdPON
            dfb3dy = CxB3 *mu*(F3 *Fexp**F2)*dFexpdPON
            dfb4dy = CxB4 *mu*(F4 *Fexp**F3)*dFexpdPON
            dfb5dy = CxB5 *mu*(F5 *Fexp**F4)*dFexpdPON
            dfb1dR = dfb1dy *dydR
            dfb2dR = dfb2dy *dydR
            dfb3dR = dfb3dy *dydR
            dfb4dR = dfb4dy *dydR
            dfb5dR = dfb5dy *dydR
            dfb1dG = dfb1dy *dydG
            dfb2dG = dfb2dy *dydG
            dfb3dG = dfb3dy *dydG
            dfb4dG = dfb4dy *dydG
            dfb5dG = dfb5dy *dydG
c
            dFggaxdR = dfa1dR+dfa2dR+dfa3dR+dfa4dR+dfa5dR +
     $                 dfb1dR+dfb2dR+dfb3dR+dfb4dR+dfb5dR
c                       
            dFggaxdG = dfa1dG+dfa2dG+dfa3dG+dfa4dG+dfa5dG +
     $                 dfb1dG+dfb2dG+dfb3dG+dfb4dG+dfb5dG
c            
             D1F(iG,dRA) = D1F(iG,dRA) + dElocdR*Fggax + Eloc*dFggaxdR
             D1F(iG,dGA) = D1F(iG,dGA) + Eloc*dFggaxdG

      EndIf  
  100 Continue
C
C     Beta contribution...
C
        Do 140 iG = 1, NGrid
        If(RB(iG).gt.TinyDFT) Then
          Rho    = RB(iG)
          Gam = D1RB(iG,1)**F2 + D1RB(iG,2)**F2 + D1RB(iG,3)**F2
          Rho43 = Rho**F4o3
          Gam12 = Sqrt(Gam)
c
          Eloc = Ax*Rho43
          x = Gam12/Rho43
          s = As*x
          y = s*s
          PON = mu*y
          Ffrac = F1-F1/(F1+PON)
          Fexp  = F1-exp(-PON)
          fa0 = CxA0
          fa1 = CxA1 *Ffrac
          fa2 = CxA2 *Ffrac**F2
          fa3 = CxA3 *Ffrac**F3
          fa4 = CxA4 *Ffrac**F4
          fa5 = CxA5 *Ffrac**F5
          fb0 = CxB0
          fb1 = CxB1 *Fexp
          fb2 = CxB2 *Fexp**F2
          fb3 = CxB3 *Fexp**F3
          fb4 = CxB4 *Fexp**F4
          fb5 = CxB5 *Fexp**F5
c
          Fggax = fa0+fa1+fa2+fa3+fa4+fa5 +
     $            fb0+fb1+fb2+fb3+fb4+fb5
c 
          F(iG)=F(iG) + Eloc*Fggax
C
C     1st derivatives.
C
            Rho13  = Rho**F1o3
            dElocdR=Ax*F4o3*Rho13
            dydR = -(F8/F3)*y/Rho
            dydG   = y/Gam
            dFfracdPON = F1/((F1+PON)**F2)
            dFexpdPON  = exp(-PON)
            dfa1dy = CxA1 *mu*dFfracdPON
            dfa2dy = CxA2 *mu*F2*Ffrac*dFfracdPON
            dfa3dy = CxA3 *mu*(F3 *Ffrac**F2)*dFfracdPON
            dfa4dy = CxA4 *mu*(F4 *Ffrac**F3)*dFfracdPON
            dfa5dy = CxA5 *mu*(F5 *Ffrac**F4)*dFfracdPON
            dfa1dR = dfa1dy *dydR
            dfa2dR = dfa2dy *dydR
            dfa3dR = dfa3dy *dydR
            dfa4dR = dfa4dy *dydR
            dfa5dR = dfa5dy *dydR
            dfa1dG = dfa1dy *dydG
            dfa2dG = dfa2dy *dydG
            dfa3dG = dfa3dy *dydG
            dfa4dG = dfa4dy *dydG
            dfa5dG = dfa5dy *dydG
            dfb1dy = CxB1 *mu*dFexpdPON
            dfb2dy = CxB2 *mu*F2*Fexp*dFexpdPON
            dfb3dy = CxB3 *mu*(F3 *Fexp**F2)*dFexpdPON
            dfb4dy = CxB4 *mu*(F4 *Fexp**F3)*dFexpdPON
            dfb5dy = CxB5 *mu*(F5 *Fexp**F4)*dFexpdPON
            dfb1dR = dfb1dy *dydR
            dfb2dR = dfb2dy *dydR
            dfb3dR = dfb3dy *dydR
            dfb4dR = dfb4dy *dydR
            dfb5dR = dfb5dy *dydR
            dfb1dG = dfb1dy *dydG
            dfb2dG = dfb2dy *dydG
            dfb3dG = dfb3dy *dydG
            dfb4dG = dfb4dy *dydG
            dfb5dG = dfb5dy *dydG
c
            dFggaxdR = dfa1dR+dfa2dR+dfa3dR+dfa4dR+dfa5dR +
     $                 dfb1dR+dfb2dR+dfb3dR+dfb4dR+dfb5dR
c                       
            dFggaxdG = dfa1dG+dfa2dG+dfa3dG+dfa4dG+dfa5dG +
     $                 dfb1dG+dfb2dG+dfb3dG+dfb4dG+dfb5dG
c            
             D1F(iG,dRB) = D1F(iG,dRB) + dElocdR*Fggax + Eloc*dFggaxdR
             D1F(iG,dGB) = D1F(iG,dGB) + Eloc*dFggaxdG
      EndIf  
  140 Continue
      Return
      End 
