      SUBROUTINE  VS98c(Tol,F,D1F,RA,RB,D1RA,D1RB,TA,TB,NGrid,ijzy)
c     ******************************************************************
c     *                                                                *
c     *  VS98c evaluates the correlation part of VS98 and its          *
c     *  variants on a grid.                                           *
c     *  !!! Second derivatives are not available yet.                 *
c     *                                                                *
c     *  Ref:  T. V. Voorhis and G. E. Scuseria, J. Chem. Phys. 109,   *
c     *        400 (1998).                                             *
c     *       Y. Zhao and D. G. Truhlar, J. Chem. Phys. 125,           * 
c     *        194101 (2006).                                          *
c     *                                                                *
c     *       ijzy - 1 correlation functional in VS98                  *
c     *       ijzy - 2 correlation functional in M06-L                 *
c     *       ijzy - 3 correlation functional in M06-HF                *
c     *       ijzy - 4 correlation functional in M06                   *
c     *       ijzy - 5 correlation functional in M06-2X                *
c     *       ijzy - 6 correlation functional in revM06-L              *
c     *       ijzy - 7 correlation functional in revM06                *
c     *       ijzy - 8 correlation functional in M06-SX                *
c     *       ijzy - 9 correlation functional in M06CR                 *
c     *                                                                *
c     *  OUTPUT:                                                       *
c     *     F      - Functional values                                 *
c     *     D1F    - First derivatives with respect to RA, RB, GA, GB  *
c     *              TA, TB                                            *
c     *                                                                *
c     *  INPUT:                                                        *
c     *     RA,B   - Spin densities                                    *
c     *     D1RA,B - Spin density gradients                            *
c     *     TA,B   - Spin kinetic energy densities                     *
c     *     NGrid  - number of grids                                   *
*                                                                      *
*  Note:  You need to code up the Perdew-Wang uniform-gas correlation  *
*          lsdac                                                       *
*  YZ (12/08)                                                          *
*                                                                      *
************************************************************************
      IMPLICIT NONE

      INTEGER NGrid
      Integer dRA, dRB, dTA, dTB, dGA, dGB
      REAL*8  F(NGrid),D1F(NGrid,7),RA(NGrid),RB(NGrid),
     $        D1RA(NGrid,3),D1RB(NGrid,3),TA(NGrid),TB(NGrid)
      REAL*8 Pi, F6, F43, Pi34, F13, 
     &RS,RSP,Zeta,dZdA,dZdB,PotLC,dLdS,dLdZ,d2LdSS,d2LdSZ,d2LdZZ,
     &  P, EUEG,Denom, DenPA, DenPB, DenGA, DenGB,EUEGPA,EUEGPB
      REAL*8 PA,GAA,TauA,FA,FPA,FGA,FTA,EUA,EUEGA,ChiA,EUPA,ChiAP,
     &ChiAG,ZA,ZAP,ZAT
      REAL*8 PB,GBB,TauB,FB,FPB,FGB,FTB,EUB,EUEGB,ChiB,EUPB,ChiBP,
     &ChiBG,ZB,ZBP,ZBT
      REAL*8 ZAB, XAB, kab, xk, zk
      REAL*8 dgdx,dgdz,dgdPA,dgdGA,dgdTA,dgdPB,dgdGB,dgdTB
      REAL*8 gcab,cf,gab
      REAL*8 r7, r8, r9, r10, r11, r12

      
      REAL*8  Tol, DTol,F1, F2, F3, F4, COpp
      Data F1/1.0d0/,F2/2.0d0/,F3/3.0d0/,F4/4.0d0/ 
     $,gab/0.00304966d0/,cf/9.115599720d0/

      INTEGER i,j,ijzy


C     Parameters for VS98 
      if (ijzy.eq.1) then
              r7=   7.035010d-01
              r8=   7.694574d-03
              r9=   5.152765d-02
              r10=   3.394308d-05
              r11=  -1.269420d-03
              r12=   1.296118d-03
C     Parameters for M06-L
      elseif (ijzy.eq.2) then
              r7=      3.957626D-01
              r8=      -5.614546D-01
              r9=      1.403963D-02
              r10=     9.831442D-04
              r11=     -3.577176D-03
              r12=     0.000000D+00
C     Parameters for M06-HF
      elseif (ijzy.eq.3) then
              r7=    -6.746338D-01
              r8=    -1.534002D-01
              r9=    -9.021521D-02
              r10=   -1.292037D-03
              r11=   -2.352983D-04
              r12=   0.000000D+00

C     Parameters for M06
      elseif (ijzy.eq.4) then
               r7= -2.741539D+00
               r8= -6.720113D-01
               r9= -7.932688D-02
               r10=1.918681D-03
               r11=-2.032902D-03
               r12=0.000000D+00

C     Parameters for M06-2X
      elseif (ijzy.eq.5) then
              r7=  1.166404D-01
              r8=  -9.120847D-02
              r9=  -6.726189D-02
              r10= 6.720580D-05
              r11= 8.448011D-04
              r12= 0.000000D+00
C     Parameters for revM06-L
      elseif (ijzy.eq.6) then
              r7=  4.007146D-01
              r8=  1.5796569D-02
              r9=  -3.2680984D-02
              r10= 0.000000D+00
              r11= 0.000000D+00
              r12= 1.260132D-03
C     Parameters for revM06
      elseif (ijzy.eq.7) then
              r7=  -3.390666720D-01
              r8=  3.790156384D-03
              r9=  -2.762485975D-02
              r10= 0.000000D+00
              r11= 0.000000D+00
              r12= 4.076285162D-04
C     Parameters for M06-SX
      elseif (ijzy.eq.8) then
              r7=  7.40594619832397D-01
              r8=  1.23306511345974D-02
              r9=  -1.88253421850249D-02
              r10=  0.000000D+00
              r11=  0.000000D+00
              r12=  4.87276242162303D-04
C     Parameters for M06CR 
      elseif (ijzy.eq.9) then
              r7=   6.18500770604554D-01
              r8=   2.00320284846066D-03
              r9=   6.83381134028813D-03
              r10=  0.000000D+00
              r11=  0.000000D+00
              r12=  1.87275718134357D-04
      endif

      Pi = F4*ATan(F1)
      F6=6.0d0
      F43 = F4 / F3
      Pi34 = F3 / (F4*Pi)
      F13 = F1 / F3
      DTol = Tol 
       
      DO i = 1,NGrid
         PA = RA(i)
         IF (PA.gt.DTol.and.TA(i).gt.DTol) THEN
            GAA   =  D1RA(i,1)**2 + D1RA(i,2)**2 + D1RA(i,3)**2 
            TauA = TA(i)

            Call vs98ss(PA,GAA,TauA,FA,FPA,FGA,FTA,EUA,ZA,
     &                ChiA,EUPA,ChiAP,ChiAG,ZAP,ZAT,ijzy)
            F(i) = F(i) + FA
            D1F(i,dRA)  = D1F(i,dRA) + FPA
            D1F(i,dGA) = D1F(i,dGA) + FGA
            D1F(i,dTA)  = D1F(i,dTA) + FTA
         ENDIF
         PB = RB(i)
         IF (PB.gt.DTol.and.TB(i).gt.DTol) THEN
            GBB   =  D1RB(i,1)**2 + D1RB(i,2)**2 + D1RB(i,3)**2
            TauB = TB(i)

            Call vs98ss(PB,GBB,TauB,FB,FPB,FGB,FTB,EUB,ZB,
     &                ChiB,EUPB,ChiBP,ChiBG,ZBP,ZBT,ijzy)
            F(i) = F(i) + FB
            D1F(i,dRB)  = D1F(i,dRB) + FPB
            D1F(i,dGB) = D1F(i,dGB) + FGB
            D1F(i,dTB)  = D1F(i,dTB) + FTB
         ENDIF
         P=PA+PB
         IF (PB.gt.DTol.and.PA.gt.DTol) THEN
          RS = (Pi34/P) ** F13 
          RSP = -RS/(F3*P)
          Zeta = (PA-PB)/P
          dZdA = (F1-Zeta)/P
          dZdB = (-F1-Zeta)/P
          Call lsdac(RS,Zeta,PotLC,dLdS,dLdZ)
          EUEG = P*PotLC - EUA - EUB
          ZAB = ZA + ZB
          XAB = ChiA+ChiB
          kab = F1 + gab*(XAB+ZAB)
          xk = XAB/kab
          zk = ZAB/kab
       call gvt4(gcab,dgdx,dgdz,xk,zk,kab,gab,r7,r8,r9,r10,r11,r12)
          F(i) = F(i) + gcab*EUEG
          dgdPA = dgdx*ChiAP + dgdz*ZAP
          dgdGA = dgdx*ChiAG
          dgdTA = dgdz*ZAT
          dgdPB = dgdx*ChiBP + dgdz*ZBP
          dgdGB = dgdx*ChiBG
          dgdTB = dgdz*ZBT
          EUEGPA = PotLC + P*dLdS*RSP + P*dLdZ*dZdA - EUPA
          EUEGPB = PotLC + P*dLdS*RSP + P*dLdZ*dZdB - EUPB
          D1F(i,dRA) = D1F(i,dRA) + (EUEGPA*gcab + EUEG*dgdPA)
          D1F(i,dRB) = D1F(i,dRB) + (EUEGPB*gcab + EUEG*dgdPB)
          D1F(i,dGA) = D1F(i,dGA) + EUEG*dgdGA 
          D1F(i,dGB) = D1F(i,dGB) + EUEG*dgdGB
          D1F(i,dTA)=  D1F(i,dTA) + EUEG*dgdTA
          D1F(i,dTB)=  D1F(i,dTB) + EUEG*dgdTB 
         ENDIF
        ENDDO
        
        RETURN
        END  
                                        

      Subroutine vs98ss(PX,GX,TX,F,FP,FG,FT,EUEG,Z,Chi,EUEGP,
     &                   ChiP,ChiG,ZP,ZT,ijzy)
      Implicit none
C
C     Compute the same-spin part of the vs98 correlation functional for one grid
C     point and one spin-case.
C

      integer ijzy
      double precision r13, r14, r15, r16, r17, r18
      double precision PX, GX, TX, F, FP, FG, FT, DTol, Z, ZP, ZT
      double precision EUEG, Chi, EUEGP, ChiP, ChiG, cf, gcc
      double precision Zero, Pt25, F1, F2, F3, F4, F5, F6, F8, F11
      double precision Pi, Pi34, F13, F23, F43, F53, F83, F113
      double precision RS, D, RSP, PotLC, DX, DZ, dgdP, dgdG, dgdT
      double precision E,DP, DG, DT, rhoo, rho43, rho53, rho83
      double precision rrho, F4o3, rho13, kc, xk, zk, gc, dgdx, dgdz
      double precision d2LdSS, d2LdSZ, d2LdZZ, dLdS, dLdZ

      Data Zero/0.0d0/, Pt25/0.25d0/, F1/1.0d0/, F2/2.0d0/, F3/3.0d0/,
     $  F4/4.0d0/, F5/5.0d0/, F6/6.0d0/, F8/8.0d0/, F11/11.0d0/,
     $  gcc/0.00515088d0/,cf/9.115599720d0/
 
 
      F4o3 = 4.0d0/3.0d0
C     Parameters for VS98 
      if (ijzy.eq.1) then
              r13=   3.270912d-01
              r14=  -3.228915d-02
              r15=  -2.942406d-02
              r16=   2.134222d-03
              r17=  -5.451559d-03
              r18=   1.577575d-02
C     Parameters for M06-L
      elseif (ijzy.eq.2) then
              r13=   4.650534D-01
              r14=   1.617589D-01
              r15=   1.833657D-01
              r16=   4.692100D-04
              r17=  -4.990573D-03
              r18=   0.000000D+00
C     Parameters for M06-HF
      elseif (ijzy.eq.3) then
              r13=   8.976746D-01
              r14=  -2.345830D-01
              r15=   2.368173D-01
              r16=  -9.913890D-04
              r17=  -1.146165D-02
              r18=   0.000000D+00
C     Parameters for M06
      elseif (ijzy.eq.4) then
               r13=  4.905945D-01
               r14= -1.437348D-01
               r15=  2.357824D-01
               r16=  1.871015D-03
               r17= -3.788963D-03
               r18=  0.000000D+00
C     Parameters for M06-2X
      elseif (ijzy.eq.5) then
              r13=  6.902145D-01
              r14=  9.847204D-02
              r15=  2.214797D-01
              r16= -1.968264D-03
              r17= -6.775479D-03
              r18=  0.000000D+00
C     Parameters for revM06-L
      elseif (ijzy.eq.6) then
              r13=  -5.38821292D-01
              r14=  -2.829603D-02
              r15=  2.3889696D-02
              r16=  0.000000D+00
              r17=  0.000000D+00
              r18=  -2.437902D-03
C     Parameters for revM06
      elseif (ijzy.eq.7) then
              r13=  -1.467095900D-01
              r14=  -1.832187007D-04
              r15=  8.484372430D-02
              r16=  0.000000D+00
              r17=  0.000000D+00
              r18=  2.280677172D-04
C     Parameters for M06-SX
      elseif (ijzy.eq.8) then
              r13=  8.17322574473352D-02
              r14= -2.88531085759385D-02
              r15=  9.05917734868130D-02
              r16=  0.000000D+00
              r17=  0.000000D+00
              r18= -4.86297499082106D-04
C     Parameters for M06CR
      elseif (ijzy.eq.9) then
              r13=  1.29365311262478D+00
              r14=  2.09071938683782D-02
              r15=  4.41700379894833D-02
              r16=  0.000000D+00
              r17=  0.000000D+00
              r18=  5.90705610127925D-04
      endif

      DTol =1.0d-8 
      If(PX.le.DTol) then
        EUEG = Zero
        Chi = Zero
        EUEGP = Zero
        ChiP = Zero
        ChiG = Zero
        PX = Zero
        GX = Zero 
        TX = Zero
        F  = Zero
        FP = Zero
        FG = Zero
        FT = Zero
        Z  = Zero
        ZP = Zero
        ZT = Zero
      else
        Pi = F4*ATan(F1)
        Pi34 = F3 / (F4*Pi)
        F13 = F1 / F3
        F23 = F2 / F3
        F43 = F2 * F23
        F53 = F5 / F3
        F83 = F8 / F3
        F113 = F11 / F3
        rhoo = PX 
        rho43 = rhoo**F4o3
        rho13 = rho43*rrho
        rho53 = rhoo**F53
        rho83 = rho53*rhoo
        
        RS = (Pi34/PX) ** F13
        Call lsdac(RS,F1,PotLC,dLdS,dLdZ)
        EUEG = PX*PotLC
        Chi = GX/rho83
        Z = (TX/rho53) - cf
        kc = F1 + gcc*(Chi + Z)
        xk = Chi/kc
        zk = Z/kc
        D = F1 - Chi/(F4*(Z + cf)) 
        call gvt4(gc,dgdx,dgdz,xk,zk,kc,gcc,r13,r14,r15,r16,r17,r18)
        E = D*EUEG*gc
        F = E 
c
        RSP = -RS/(F3*Px)
        ChiG = F1/PX**F83
        ChiP = -F83*Chi/PX
        ZP = -F53 * TX/rho83
        ZT =  F1/rho53
        DZ = Chi/(F4*(Z + cf)*(Z + cf)) 
        DX = -F1/(F4*(Z + cf))
        DP = DZ*ZP + DX*ChiP
        DG = DX*ChiG
        DT = DZ*ZT
        dgdP = dgdx*ChiP + dgdz*ZP
        dgdG = dgdx*ChiG 
        dgdT = dgdz*ZT
        EUEGP = PotLC + PX*dLdS*RSP
        FP = DP*EUEG*gc + D*EUEGP*gc + D*EUEG*dgdP
        FG = DG*EUEG*gc + D*EUEG*dgdG
        FT = DT*EUEG*gc + D*EUEG*dgdT
       Endif
       Return
       End
 

